@@ -27,18 +27,18 @@ module Version = Versioning.Debian
 let label =  __label ;;
 include Util.Logging(struct let label = label end) ;;
 
-let max32int = Int32.to_int(Int32.max_int);;
-
 module SMap = Map.Make (String)
 
 type tables = {
-  virtual_table : (string,SSet.t ref) OcamlHashtbl.t; (** all names of virtual packages *)
-  unit_table : unit Util.StringHashtbl.t ;         (** all names of real packages    *)
-  versions_table : int Util.StringHashtbl.t;       (** version -> int table          *)
+  virtual_table : (string,SSet.t ref) OcamlHashtbl.t; (** all names of virtual packages    *)
+  unit_table : unit Util.StringHashtbl.t ;            (** all names of real packages       *)
+  versions_table : int Util.StringHashtbl.t;          (** version -> int table             *)
   reverse_table : (string SMap.t) ref Util.IntHashtbl.t;
-  versioned_table : unit Util.StringHashtbl.t;     (** all (versions,name) tuples. 
-                                                       all versions mentioned of depends, 
-                                                       pre_depends, conflict and breaks    *)
+  versioned_table : unit Util.StringHashtbl.t;        (** all (versions,name) tuples. 
+                                                        all versions mentioned of depends, 
+                                                        pre_depends, conflict and breaks   *)
+  essential_table:
+    (string,Packages.package list ref) Hashtbl.t;     (** name -> packages list            *)
 }
 
 let create n = {
@@ -47,6 +47,7 @@ let create n = {
   versions_table = Util.StringHashtbl.create (10 * n);
   versioned_table = Util.StringHashtbl.create (10 *n);
   reverse_table = Util.IntHashtbl.create (10 * n);
+  essential_table = Hashtbl.create (n / 100);
 }
 
 type lookup = {
@@ -63,6 +64,8 @@ type options = {
   host : string option;   (* the host architecture - cross compile *)
   ignore_essential : bool;
   builds_from : bool;
+  drop_bd_indep : bool;
+  profiles : string list; (* list of active build profiles *)
 }
 
 let default_options = {
@@ -72,6 +75,8 @@ let default_options = {
   host = None;
   ignore_essential = false;
   builds_from = false;
+  drop_bd_indep = false;
+  profiles = [];
 }
 
 let add_name_arch n = function
@@ -105,63 +110,59 @@ let clear tables =
   Util.StringHashtbl.clear tables.unit_table;
   Util.StringHashtbl.clear tables.versions_table;
   Util.StringHashtbl.clear tables.versioned_table;
-  Util.IntHashtbl.clear tables.reverse_table
+  Util.IntHashtbl.clear tables.reverse_table;
+  Hashtbl.clear tables.essential_table;
 ;;
 
-let add table k v =
+(* string version -> int version Hash *)
+let add_v tables k v =
+  if not(Util.StringHashtbl.mem tables.versions_table k) then
+    Util.StringHashtbl.add tables.versions_table k v
+
+(* package names Set *)
+let add_p table k =
   if not(Util.StringHashtbl.mem table k) then
-    Util.StringHashtbl.add table k v
+    Util.StringHashtbl.add table k ()
 
-let add_v table k v =
-  if not(Util.StringPairHashtbl.mem table k) then
-    Util.StringPairHashtbl.add table k v
+(* (version,name) Set *)
+let add_pairs table (v,p) =
+  if not(Util.StringPairHashtbl.mem table (v,p)) then
+    Util.StringPairHashtbl.add table (v,p) ()
 
+(* name -> (name,constr) Hash *)
 let add_s h k v =
   try let s = OcamlHashtbl.find h k in s := SSet.add v !s
   with Not_found -> OcamlHashtbl.add h k (ref (SSet.singleton v))
 ;;
 
 (* collect names of virtual packages 
- * associates to each virtual package a list of real packages *)
-let init_virtual_table table pkg =
+   associates to each virtual package a list of real packages *)
+let init_virtual_table tables pkg =
   List.iter (fun ((name,_),constr) -> 
-    add_s table name (pkg#name,constr)
-    (*
-    match CudfAdd.cudfop constr with
-    |None -> add_s table name (pkg#name,None)
-    |Some(_,version) as c -> add_s table name (pkg#name,c)
-  *)
+    add_s tables.virtual_table name (pkg#name,constr)
   ) pkg#provides
 
 (* collect names of real packages *)
-let init_unit_table table pkg = 
-  add table pkg#name ()
-
-(* collect all versions mentioned of depends, pre_depends, conflict and breaks *)
-let init_versioned_table table pkg =
-  let conj_iter l = List.iter (fun ((name,_),_)-> add table name ()) l in
-  let cnf_iter ll = List.iter conj_iter ll in
-  conj_iter pkg#conflicts ;
-  conj_iter pkg#breaks ;
-  cnf_iter pkg#pre_depends ;
-  cnf_iter pkg#depends
-;;
+let init_unit_table tables pkg = 
+  add_p tables.unit_table pkg#name
 
 (* collect all versions mentioned anywhere in the universe, including source fields *)
-let init_versions_table table pkg =
+let init_versions_table tables table pkg =
+  let versionedtables = tables.versioned_table in
   let conj_iter l =
     List.iter (fun ((name,_),constr) ->
-      match CudfAdd.cudfop constr with
+      add_p versionedtables name;
+      match constr with
       |None -> ()
-      |Some(_,version) -> add_v table (version,name) ()
+      |Some(_,version) -> add_pairs table (version,name)
     ) l
   in
   let cnf_iter ll = List.iter conj_iter ll in
   let add_source pv = function
-    |(p,None) -> add_v table (pv,p) ()
-    |(p,Some(v)) -> add_v table (v,p) ()
-  in 
-  add_v table (pkg#version,pkg#name) ();
+    |(p,None) -> add_pairs table (pv,p)
+    |(p,Some(v)) -> add_pairs table (v,p)
+  in
+  add_pairs table (pkg#version,pkg#name);
   conj_iter pkg#breaks;
   conj_iter pkg#provides;
   conj_iter pkg#conflicts ;
@@ -172,21 +173,21 @@ let init_versions_table table pkg =
   add_source pkg#version pkg#source
 ;;
 
-let init_tables ?(step=1) ?(versionlist=[]) pkglist =
+let init_tables ?(options=default_options) ?(step=1) ?(versionlist=[]) pkglist =
   let n = List.length pkglist in
   let tables = create n in 
   let temp_versions_table = Util.StringPairHashtbl.create (10 * n) in
-  let ivrt = init_virtual_table tables.virtual_table in
-  (* XXX init_versions_table and init_versioned_table can be done at the same time !!! *)
-  let ivt = init_versions_table temp_versions_table in
-  let ivdt = init_versioned_table tables.versioned_table in
-  let iut = init_unit_table tables.unit_table in
-
-  List.iter (fun v -> add_v temp_versions_table (v,"") ()) versionlist; 
-  List.iter (fun pkg -> ivt pkg ; ivrt pkg ; ivdt pkg ; iut pkg) pkglist ;
+  let ivrt = init_virtual_table tables in
+  let ivt = init_versions_table tables temp_versions_table in
+  let iut = init_unit_table tables in
+  let iet pkg =
+    if not options.ignore_essential && pkg#essential then
+      CudfAdd.add_to_package_list tables.essential_table (pkg#name) pkg
+  in
+  List.iter (fun v -> add_pairs temp_versions_table (v,"")) versionlist; 
+  List.iter (fun pkg -> iet pkg; ivt pkg; ivrt pkg ; iut pkg) pkglist ;
   let l = Util.StringPairHashtbl.fold (fun v _ acc -> v::acc) temp_versions_table [] in
   let add_reverse i (n,v) =
-    debug "Add Reverse (%s,%s) %i" n v i;
     try 
       let m = Util.IntHashtbl.find tables.reverse_table i in 
       m := (SMap.add n v !m)
@@ -198,13 +199,12 @@ let init_tables ?(step=1) ?(versionlist=[]) pkglist =
   let rec numbers (prec,i) = function
     |[] -> ()
     |(v,n)::t ->
-      debug "Numbers %s %s" n v;
       if Version.equal v prec then begin
-        add tables.versions_table v i;
+        add_v tables v i;
         if n <> "" then add_reverse i (n,v);
         numbers (prec,i) t
       end else begin
-        add tables.versions_table v (i+step);
+        add_v tables v (i+step);
         if n <> "" then add_reverse (i+step) (n,v);
         numbers (v,(i+step)) t
       end
@@ -221,24 +221,25 @@ let get_cudf_version tables (package,version) =
     raise Not_found
   end
 
-let get_real_version tables (name,cudfversion) =
-  let package =
-    (* XXX this is a hack. Remove --virtual- and architecture encoding *)
-    let n = (CudfAdd.decode name) in
-    if ExtString.String.starts_with n "--vir" then
-      ExtString.String.slice ~first:10 n
-    else
-      try
-        let (n,a) = ExtString.String.split n ":" in
-        if n = "src" then a else n
-      with Invalid_string -> n
-  in
+let get_real_name name = 
+  (* Remove --virtual- and architecture encoding *)
+  let dn = (CudfAdd.decode name) in
+  if ExtString.String.starts_with dn "--vir" then
+    (ExtString.String.slice ~first:10 dn,None)
+  else
+    try
+      let (n,a) = ExtString.String.split dn ":" in
+      if n = "src" then (a,None) else (n,Some a)
+    with Invalid_string -> (dn,None)
+
+let get_real_version tables (cudfname,cudfversion) =
+  let (debname,arch) = get_real_name cudfname in
   try
-    if cudfversion = max32int || cudfversion = max32int - 1 then 
-      (package,"nan")
+    if cudfversion = Util.max32int || cudfversion = Util.max32int - 1 then 
+      (debname,arch,"nan")
     else
       let m = !(Util.IntHashtbl.find tables.reverse_table cudfversion) in
-      try (package,SMap.find package m) 
+      try (debname,arch,SMap.find debname m) 
       with Not_found ->
         let known =
           String.concat "," (
@@ -247,30 +248,31 @@ let get_real_version tables (name,cudfversion) =
             ) (SMap.bindings m)
           )
         in
-        fatal "Unable to get real version for %s (%i)\n All Known versions for this package are %s" package cudfversion known 
+        fatal "Unable to get real version for %s (%i)\n All Known versions for this package are %s" debname cudfversion known 
   with Not_found ->
-    fatal "Package (%s,%d) does not have an associated debian version" name cudfversion
+    fatal "Package (%s,%d) does not have an associated debian version" cudfname cudfversion
 
 let loadl ?native_arch ?package_arch tables l =
   List.flatten (
     List.map (fun ((name,_) as vpkgname,constr) ->
       let encname = add_arch_info ?native_arch ?package_arch vpkgname in
-      match CudfAdd.cudfop constr with
+      match constr with
       |None ->
           (* Versioned virtual packages will satisfiy non versioned dependencies *)
           if (OcamlHashtbl.mem tables.virtual_table name) then
-            [(encname, None);("--virtual-"^encname,Some(`Eq,max32int - 1))]
+            [(encname, None);("--virtual-"^encname,Some(`Eq,Util.max32int - 1))]
           else
             [(encname, None)]
       |Some(op,v) ->
           (* Non-versioned virtual packages will not satisfy versioned dependencies. *)
+          let op = Pef.Pefcudf.pefcudf_op op in
           try
             match SSet.elements !(OcamlHashtbl.find tables.virtual_table name) with
             |[] -> assert false
             |l ->
                 let dl = 
                   List.filter_map (function
-                    |(_,None) -> Some ("--virtual-"^encname,Some(`Eq,max32int))
+                    |(_,None) -> Some ("--virtual-"^encname,Some(`Eq,Util.max32int))
                     |(_,Some _) ->
                         let constr = Some(op,get_cudf_version tables (name,v)) in
                         Some ("--virtual-"^encname,constr)
@@ -302,9 +304,9 @@ let loadlp ?native_arch ?package_arch tables l =
   List.flatten (
     List.map (fun ((name,_) as vpkgname,constr) ->
       let encname = add_arch_info ?native_arch ?package_arch vpkgname in
-      match CudfAdd.cudfop constr with
-      |None -> [("--virtual-"^encname,Some(`Eq,max32int - 1))]
-      |Some(`Eq,v) ->
+      match constr with
+      |None -> [("--virtual-"^encname,Some(`Eq,Util.max32int - 1))]
+      |Some("=",v) ->
         let constr = Some(`Eq,get_cudf_version tables (name,v)) in
         [("--virtual-"^encname,constr);(encname,constr)]
       |_ -> fatal "This should never happen : a provide can be either = or unversioned"
@@ -329,7 +331,6 @@ let preamble =
     ("sourcenumber",(`String (Some "")));
     ("sourceversion",(`Int (Some 1))) ;
     ("essential",(`Bool (Some false))) ;
-    ("buildessential",(`Bool (Some false))) ;
     ("filename",(`String (Some "")));
     ("installedsize",(`Int (Some 0)));
     ("multiarch",(`String (Some "")));
@@ -340,16 +341,21 @@ let preamble =
 
 let add_extra ?native_arch extras tables pkg =
   let name = 
-    if String.starts_with pkg#name "src:" then
-      String.sub pkg#name 4 ((String.length pkg#name) - 4)
-    else pkg#name
+    let n = 
+      if String.starts_with pkg#name "src:" then
+        String.sub pkg#name 4 ((String.length pkg#name) - 4)
+      else pkg#name
+    in
+    Some ("name",`String n) 
+  in
+  let number = Some ("number",`String pkg#version) in
+  let architecture = Some ("architecture",`String pkg#architecture) in
+  let priority = Some ("priority",`String pkg#priority) in
+  let essential =
+    if pkg#essential then 
+      Some ("essential", `Bool true) 
+    else None 
   in
-  let name = ("name",`String name) in
-  let number = ("number",`String pkg#version) in
-  let architecture = ("architecture",`String pkg#architecture) in
-  let priority = ("priority",`String pkg#priority) in
-  let essential = ("essential", `Bool pkg#essential) in
-  let build_essential = ("buildessential", `Bool pkg#build_essential) in
   let (source,sourcenumber,sourceversion) =
     let (n,v) =
       match pkg#source with
@@ -358,16 +364,25 @@ let add_extra ?native_arch extras tables pkg =
       |(n,Some v) -> (n,v)
     in
     let cv = get_cudf_version tables ("",v) in
-    ("source",`String n), ("sourcenumber", `String v), ("sourceversion", `Int cv)
+    Some ("source",`String n),
+    Some ("sourcenumber", `String v),
+    Some ("sourceversion", `Int cv)
   in
-  let recommends = ("recommends", `Vpkgformula (loadll tables pkg#recommends)) in
-  let replaces = ("replaces", `Vpkglist (loadl tables pkg#replaces)) in
-  let native = (* XXX if 0 we can ignore it *)
-    let n =
-      if Option.is_some native_arch && 
-        (Option.get native_arch) = pkg#architecture then 1 
-      else 0
-    in ("native",`Int n)
+  let recommends = 
+    match loadll tables pkg#recommends with
+    |[] -> None
+    |l -> Some ("recommends", `Vpkgformula l) 
+  in
+  let replaces = 
+    match loadl tables pkg#replaces with
+    |[] -> None
+    |l -> Some ("replaces", `Vpkglist l)
+  in
+  let native =
+    if Option.is_some native_arch && 
+      (Option.get native_arch) = pkg#architecture 
+    then Some ("native",`Int 1)
+    else None
   in
   let extras = 
     ("Type",("type",`String None))::
@@ -388,27 +403,17 @@ let add_extra ?native_arch extras tables pkg =
     ) extras
   in
   List.filter_map (function
-    |(_,`Vpkglist []) -> None
-    |(_,`Vpkgformula []) -> None
-    |(_,`String "") -> None
-    |e -> Some e
+    |Some (_,`Vpkglist []) -> None
+    |Some (_,`Vpkgformula []) -> None
+    |Some (_,`String "") -> None
+    |e -> e
   )
   [priority; name; architecture; number;
   source; sourcenumber; sourceversion; 
-  recommends; replaces;
-  essential;build_essential;native]@ l
+  recommends; replaces; essential;native]@ l
 ;;
 
-let set_keep pkg =
-  match pkg#essential,Packages.is_on_hold pkg with
-  |_,true -> `Keep_version
-  |true,_ -> `Keep_package
-  |false,_ -> `Keep_none
-;;
-
-let add_inst inst pkg =
-  if inst then true 
-  else Packages.is_installed pkg
+let add_inst inst pkg = inst || Packages.is_installed pkg 
 
 let tocudf tables ?(options=default_options) ?(inst=false) pkg =
   let bind m f = List.flatten (List.map f m) in
@@ -511,11 +516,11 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
                       end
                   |false, true ->             (* a virtual package and a self conflict *)
                       begin
-                        debug "M-A-Same: pkg %s has a self-conflict via virtual package: %s" pkg#name n;
+                        (*debug "M-A-Same: pkg %s has a self-conflict via virtual package: %s" pkg#name n;*)
                         try
                           List.filter_map (fun (pn,_) ->
                             if pn <> pkg#name then begin
-                              debug "M-A-Same: adding conflict on real package %s for %s" pn pkg#name; 
+                              (*debug "M-A-Same: adding conflict on real package %s for %s" pn pkg#name; *)
                               Some((pn,a),None)
                             end else None
                           ) (SSet.elements !(OcamlHashtbl.find tables.virtual_table n))
@@ -531,19 +536,18 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
     let _depends = 
       loadll ~native_arch ~package_arch tables (pkg#pre_depends @ pkg#depends)
     in
-    (* XXX: if ignore essential for the moment we also ignore keep *)
-    let _keep =
-      if options.ignore_essential then
-        `Keep_none
-      else if (package_arch <> native_arch && package_arch <> "all") then
+    let _keep pkg =
+      if (package_arch <> native_arch && package_arch <> "all") then
         `Keep_none
+      else if Packages.is_on_hold pkg && Packages.is_installed pkg then
+        `Keep_version
       else
-        set_keep pkg
+        `Keep_none
     in
     { Cudf.default_package with
       Cudf.package = _name ;
       Cudf.version = _version ;
-      Cudf.keep = _keep ;
+      Cudf.keep = _keep pkg ;
       Cudf.depends = _depends ;
       Cudf.conflicts = _conflicts ;
       Cudf.provides = _provides ;
@@ -561,7 +565,7 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
     { Cudf.default_package with
       Cudf.package = encpkgname ;
       Cudf.version = get_cudf_version tables (pkg#name,pkg#version) ;
-      Cudf.keep = if options.ignore_essential then `Keep_none else set_keep pkg;
+      Cudf.keep = if Packages.is_on_hold pkg then `Keep_version else `Keep_none ;
       Cudf.depends = loadll tables (pkg#pre_depends @ pkg#depends); 
       Cudf.conflicts = loadlc tables pkg#name (pkg#breaks @ pkg#conflicts) ;
       Cudf.provides = loadlp tables pkg#provides ;
@@ -569,10 +573,15 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
       Cudf.pkg_extra = add_extra options.extras_opt tables pkg ;
     }
 
+let get_essential ?(options=default_options) tables =
+  Hashtbl.fold (fun name { contents = l } acc ->
+    ([(name,None)],List.map (tocudf tables ~options) l)::acc
+  ) tables.essential_table []
+
 let load_list ?options l =
   let timer = Util.Timer.create "Debian.ToCudf" in
   Util.Timer.start timer;
-  let tables = init_tables l in
+  let tables = init_tables ?options l in
   let pkglist = List.map (tocudf tables ?options) l in
   clear tables;
   Util.Timer.stop timer pkglist
